# -*- coding: utf-8 -*-
"""
    drupan.template

    Integrate the jinja templating engine.
"""

from jinja2 import FileSystemLoader, Environment


def filter_more(content):
    """
    if there is a more tag return everything above else return the content
    string
    """
    (first, more, second) = content.partition("<!--MORE-->")

    if more == "<!--MORE-->":
        return first
    else:
        return content


def filter_filter(site, key, value):
    """
    returns one or more entities for a specific key / value combination. Uses
    Site.get()
    """
    filtered = site.get(key, value)

    if type(filtered) != list:
        return [filtered]

    return filtered


def filter_get(site, key, value):
    """
    returns one entity for a specific key / value combination. Uses Site.get()
    """
    filtered = site.get(key, value)

    if type(filtered) == list:
        raise Exception("too many results", key, value)
    if filtered is None:
        raise Exception("no result for", key, value)

    return filtered


class Render(object):
    """render site using Jinja"""
    def __init__(self, site, config):
        """
        Arguments:
            site: instance of drupan.site.Site
            config: instance of drupan.config.Config
        """
        self.site = site
        self.config = config

        self.template = config.get_option("jinja", "template")

    def run(self):
        """run the plugin"""
        env = Environment(
            loader=FileSystemLoader(self.template),
            extensions=["jinja2.ext.with_"]
        )

        env.filters["more"] = filter_more
        env.filters["filter"] = filter_filter
        env.filters["get"] = filter_get

        for page in self.site.entities:
            if not page.layout:
                continue

            name = "_{0}.html".format(page.layout)
            template = env.get_template(name)
            page.rendered = template.render(
                obj=page,
                site=self.site,
                config=self.config
            )



# -*- coding: utf-8 -*-
"""
    drupan.deployment.gitsub

    Deploy your site via git. Git is called via subprocess.
"""

import datetime
import shlex
import subprocess


CMD_ADD = 'git add .'
CMD_COMMIT = 'git commit -a -m "%s"' % str(datetime.datetime.now())
CMD_PUSH = 'git push -u server master'


class Deploy(object):
    def __init__(self, site, config):
        """
        Arguments:
            site: generated site
            config: config for this site
        """
        self.site = site
        self.config = config

        self.path = config.get_option("gitsub", "path")

    def execute(self, command):
        """
        Execute a command

        Arugments:
            command: command to execute
        """
        proc = subprocess.Popen(shlex.split(command), cwd=self.path)
        proc.communicate()

    def run(self):
        """run the deployment process"""
        self.execute(CMD_ADD)
        self.execute(CMD_COMMIT)
        self.execute(CMD_PUSH)



# -*- coding: utf-8 -*-
"""
    drupan.engine

    The engine is part of drupan that holds all plugins and subsystems
    together and runs the actual site creation process.
"""

from .site import Site
from .config import Config
from .template import Render
from .serve import http


class Engine(object):
    """Drupan engine doing all the work"""
    def __init__(self):
        self.site = Site()
        self.config = Config()
        self.reader = None
        self.writer = None
        self.plugins = list()
        self.renderer = None
        self.deployment = None

    def prepare_engine(self):
        """get all subsystems and plugins setup"""
        self.reader = __import__(
            "drupan.inout.{0}".format(self.config.reader),
            fromlist=["Reader"]
        ).Reader(self.site, self.config)

        self.writer = __import__(
            "drupan.inout.{0}".format(self.config.writer),
            fromlist=["Writer"]
        ).Writer(self.site, self.config)

        for name in self.config.plugins:
            plugin = __import__(
                "drupan.plugins.{0}".format(name),
                fromlist=["Plugin"]
            ).Plugin(self.site, self.config)
            self.plugins.append(plugin)

        self.renderer = Render(self.site, self.config)

        if self.config.deployment:
            self.deployment = __import__(
                "drupan.deployment.{0}".format(self.config.deployment),
                fromlist=["Deploy"]
            ).Deploy(self.site, self.config)

    def run(self):
        """run the site generation process"""
        self.reader.run()
        for plugin in self.plugins:
            plugin.run()
        self.renderer.run()
        self.writer.run()

    def serve(self):
        """serve the generated site"""
        http(self.config.get_option("writer", "directory"))

    def deploy(self):
        """deploy the generated site"""
        if not self.deployment:
            return

        self.deployment.run()



# -*- coding: utf-8 -*-
import argparse
import sys
import os

from .engine import Engine
from .version import __version__


WRONG_VERSION = """Your config parameter is a directory. If you just updated
drupan your site is likely still configured for drupan 1, you are using
drupan 2.
Please check the migration guide in the docs directory on
<https://github.com/fallenhitokiri/drupan/docs/migration_1_to_2.md>
"""

INIT = """For a new site you can clone \
<https://github.com/fallenhitokiri/drupan-template-blog> \
which should give you a good starting point and should always be up to date \
with the latest chages.

After cloning switch to the directory and run `drupan config.yaml --serve`
"""


def cmd():
    """command line entry point"""
    desc = "drupan v{0}".format(__version__)
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument("config", help="configuration file")
    parser.add_argument(
        "--nodeploy", help="do not deploy", action="store_true", default=False
    )
    parser.add_argument(
        "--serve", help="serve site", action="store_true", default=False
    )
    parser.add_argument(
        "--init",
        help="show how to setup a new site",
        action="store_true",
        default=False
    )
    parser.add_argument(
        "--deploy",
        help="deploy without generating the site",
        action="store_true",
        default=False
    )

    if "--init" in sys.argv:
        print INIT
        sys.exit()

    args = parser.parse_args()

    engine = Engine()

    # if someone uses a directory as config it is likely a drupan 1 setup
    # point to the migration guide.
    if os.path.isdir(args.config):
        print WRONG_VERSION
        sys.exit()

    engine.config.from_file(args.config)
    engine.prepare_engine()

    if args.deploy:
        engine.deploy()
        sys.exit()

    engine.run()

    if args.serve:
        engine.serve()

    if not args.nodeploy:
        engine.deploy()



# -*- coding: utf-8 -*-
"""
    drupan.deployment.s3cf

    Deploying the website to AWS S3 and CloudFront. This plugin will send
    and invalidation request to CloudFront once the deployment is complete,
    invalidating *all* files that changed.

    Using CloudFront invalidation requests can result in high bills. Please
    consider this before using this plugin. If this is a deal breaker use
    s3sub with a 5 minute cache policy.

    Requirement:
        Filesystem writer
"""

import os
from hashlib import md5
import json
from io import open

import boto
from boto.s3.connection import S3Connection
from boto.s3.key import Key


class Deploy(object):
    def __init__(self, site, config):
        """
        Arguments:
            site: generated site
            config: config for this site
        """
        self.site = site
        self.config = config

        self.bucket_name = config.get_option("s3cf", "bucket")
        self.path = config.get_option("writer", "directory")
        self.md5_path = config.get_option("s3cf", "md5path")
        self.redirects = config.get_option("s3cf", "redirects")
        self.skip_upload = config.get_option("s3cf", "skip_upload")
        self.aws_access_key = config.get_option("s3cf", "aws_access_key")
        self.aws_secret_key = config.get_option("s3cf", "aws_secret_key")
        self.cloudfront_id = config.get_option("s3cf", "cloudfront_id")

        self.new_md5s = dict()
        self.old_md5s = dict()
        self.changed = list()
        self.s3_connection = None
        self.bucket = None
        self.cf_connection = None

    def run(self):
        """run the deployment process"""
        self.setup()
        self.upload_files()
        self.invalidate()

        # reset - we use the same variables again
        self.new_md5s = dict()
        self.old_md5s = dict()
        self.changed = list()

        self.upload_redirects()

    def setup(self):
        """setup AWS connection"""
        self.s3_connection = S3Connection(
            self.aws_access_key,
            self.aws_secret_key,
        )
        self.bucket = self.s3_connection.get_bucket(self.bucket_name)
        self.cf_connection = boto.connect_cloudfront(
            self.aws_access_key,
            self.aws_secret_key,
        )

    def upload_redirects(self):
        """upload redirects for S3"""
        md5_file = "redirects.md5.json"
        # return if there are no redirects
        if self.redirects == dict():
            return

        self.generate_redirect_md5s()
        self.load_md5(md5_file)
        self.compare_md5s()
        self.save_md5(md5_file)
        self.redirect()

    def upload_files(self):
        """upload generated site to S3"""
        md5_file = "content.md5.json"
        self.generate_site_md5s()
        self.load_md5(md5_file)
        self.compare_md5s()
        self.save_md5(md5_file)

        if self.should_upload:
            self.upload()

    def generate_site_md5s(self):
        """generate MD5 checksums for all entities"""
        for root, dirs, files in os.walk(self.path):
            for name in files:
                # ignore hidden files
                if name.startswith("."):
                    continue

                filepath = os.path.join(root, name)

                with open(filepath, "rb") as infile:
                    raw = infile.read()
                    self.new_md5s[filepath] = md5(raw).hexdigest()

    def generate_redirect_md5s(self):
        """generate MD5 checkusm for all redirects"""
        for redirect in self.redirects:
            self.new_md5s[redirect] = md5(self.redirects[redirect]).hexdigest()

    def load_md5(self, filename):
        """
        load MD5 checksums from disk (json format)

        Arguments:
            filename: name of the file to load MD5 sums from
        """
        load = os.path.join(self.md5_path, filename)

        # if there is no MD5 file, just return
        if not os.path.isfile(load):
            return

        with open(load, "r", encoding="utf-8") as infile:
            raw = infile.read()
            self.old_md5s = json.loads(raw)

    def save_md5(self, filename):
        """
        save MD5 checksums to disk (json format)

        Arguments:
            filename: name of the file to save MD5 sums to
        """
        save = os.path.join(self.md5_path, filename)
        with open(save, "w", encoding="utf-8") as outfile:
            dumped = json.dumps(self.new_md5s)
            outfile.write(unicode(dumped))

    def compare_md5s(self):
        """compare MD5 checksums"""
        for key in self.new_md5s:
            if not key in self.old_md5s:
                self.changed.append(key)
                continue

            if self.new_md5s[key] != self.old_md5s[key]:
                self.changed.append(key)

    def upload(self):
        """upload changed files to S3"""
        for local in self.changed:
            remote = local.replace(self.path, "")
            key = Key(self.bucket)
            key.key = remote
            key.set_contents_from_filename(local)
            key.set_acl('public-read')
            print "uploading: {1}".format(local, remote)

    def invalidate(self):
        invalid = list()

        for local in self.changed:
            url = local.replace(self.path, "")
            invalid.append(url)

        self.cf_connection.create_invalidation_request(
            self.cloudfront_id,
            invalid
        )
        print "invalidating: {0}".format(invalid)

    @property
    def should_upload(self):
        """
        Check if a changed file is part of self.skip_upload. If this
        is the case for all changed files do not upload the site.

        Returns:
            True if the site should be uploaded
        """
        upload = False

        for changed in self.changed:
            url = changed.replace(self.path, "")

            if not url in self.skip_upload:
                upload = True

        return upload

    def redirect(self):
        """create a redirect"""
        for redirect in self.changed:
            # lstrip to make sure joining works if redirect starts with a /
            source = redirect
            destination = self.redirects[redirect]

            key = Key(self.bucket)
            key.key = source
            key.set_redirect(destination)
            key.set_acl('public-read')
            print "redirecting: {0} -> {1}".format(source, destination)



# -*- coding: utf-8 -*-
"""
    drupan.io.filesystem

    Filesystem implememnts a filesystem reader and writer. It reads drupans
    content files from the your disk and writes the generated site back to it.
"""

import os
from io import open
import shutil
import errno
from HTMLParser import HTMLParser

import yaml

from drupan.entity import Entity


class Reader(object):
    """
    Implements a filesystem reader parsing drupans standard format. It consists
    of a header with all meta data, separated by three dashs from the content.
    """
    def __init__(self, site, config):
        """
        Arguments:
            site: instance of drupan.site.Site
            config: instance of drupan.config.Config
        """
        self.site = site
        self.config = config
        self.directory = config.get_option("reader", "directory")
        self.extension = config.get_option("reader", "extension")

        if not self.extension.startswith("."):
            self.extension = ".{0}".format(self.extension)

    def run(self):
        """read and parse content files"""
        for current in os.listdir(self.directory):
            if not os.path.splitext(current)[1] == self.extension:
                continue

            fqp = os.path.join(self.directory, current)

            with open(fqp, 'r', encoding='utf-8') as infile:
                self.parse_file(infile.read())

    def parse_file(self, raw):
        """
        create an entity from a read file

        Arguments:
            raw: input
        """
        (header, seperator, content) = raw.partition("---")
        meta = yaml.load(header)

        entity = Entity(self.config)
        entity.meta = meta
        entity.raw = content.strip()

        self.site.entities.append(entity)


class Writer(object):
    """
    Implements a filesystem writer, writing all entities which have a rendered
    attribute != None. The path is the URL split and appended to the base path.
    """
    def __init__(self, site, config):
        """
        Arguments:
            site: instance of drupan.site.Site
            config: instance of drupan.config.Config
        """
        self.site = site
        self.config = config

        self.base_path = config.get_option("writer", "directory")
        self.template = config.get_option("jinja", "template")

    def run(self):
        """run the plugin"""
        self.cleandir()
        self.copytree()

        for entity in self.site.entities:
            if entity.path is None:
                continue

            self.create_path(entity)
            self.write(entity)
            self.copy_images(entity)

    def cleandir(self):
        """clean the output directory"""
        try:
            shutil.rmtree(self.base_path)
        except:
            pass

    def copytree(self):
        """
        copy the template directory to the output directory, skip files
        prefixed with _
        """
        ignore = shutil.ignore_patterns("_*")
        shutil.copytree(self.template, self.base_path, ignore=ignore)

    def create_path(self, entity):
        """
        Create all directories needed to write an entity.

        Arguments:
            entity: entity to write (drupan.entity.Entity)
        """
        if entity.path == "":
            return

        path = os.path.join(self.base_path, entity.path)

        if os.path.exists(path):
            return

        try:
            os.makedirs(path)
        except OSError:
            if OSError.errno == errno.EEXIST:
                pass
            else:
                raise

    def write(self, entity):
        """
        Write an entity to disk

        Arguments:
            entity: entity to write (drupan.entity.Entity)
        """
        path = os.path.join(self.base_path, entity.path, "index.html")

        with open(path, "w", encoding="utf-8") as output:
            output.write(entity.rendered)

    def copy_images(self, entity):
        """
        Copy images that are linked in this entity to the entity folder

        Arguments:
            entity: entity to copy images to
        """
        if entity.rendered is None:
            return

        parser = ImageParser()
        parser.feed(entity.rendered)

        # no images found?
        if len(parser.images) == 0:
            return

        path = os.path.join(self.base_path, entity.path)
        source = os.path.join(
            self.config.get_option("reader", "directory"),
            "images"
        )

        for linked in parser.images:
            origin = os.path.join(source, linked)
            shutil.copy(origin, path)


class ImageParser(HTMLParser):
    """Handler based on HTMLParser for images"""
    def __init__(self):
        self.images = []
        HTMLParser.__init__(self)

    def handle_starttag(self, tag, attrs):
        """if image tag is found add it to self.images"""
        if tag == 'img':
            self.images.append((dict(attrs)['src']))



# -*- coding: utf-8 -*-
"""
    drupan.deployment.s3sub

    Deploy your site to AWS S3. The AWS cli tool is called via subprocess.

    Requirement:
        Filesystem writer
"""

import subprocess
from hashlib import md5
import json
from io import open
import os
from urlparse import urljoin


class Deploy(object):
    def __init__(self, site, config):
        """
        Arguments:
            site: generated site
            config: config for this site
        """
        self.site = site
        self.config = config

        self.bucket = config.get_option("s3sub", "bucket")
        self.profile = config.get_option("s3sub", "profile")
        self.path = config.get_option("writer", "directory")
        self.md5_path = config.get_option("s3sub", "md5path")
        self.redirects = config.get_option("s3sub", "redirects")
        self.site_url = config.get_option("s3sub", "site_url")
        self.skip_upload = config.get_option("s3sub", "skip_upload")

        self.new_md5s = dict()
        self.old_md5s = dict()
        self.changed = list()
        self.s3path = "s3://{0}".format(self.bucket)

    def run(self):
        """run the deployment process"""
        self.upload_files()

        # reset - we use the same variables again
        self.new_md5s = dict()
        self.old_md5s = dict()
        self.changed = list()

        self.upload_redirects()

    def upload_redirects(self):
        """upload redirects for S3"""
        md5_file = "redirects.md5.json"
        # return if there are no redirects
        if self.redirects == dict():
            return

        self.generate_redirect_md5s()
        self.load_md5(md5_file)
        self.compare_md5s()
        self.save_md5(md5_file)
        self.redirect()

    def upload_files(self):
        """upload generated site to S3"""
        md5_file = "content.md5.json"
        self.generate_site_md5s()
        self.load_md5(md5_file)
        self.compare_md5s()
        self.save_md5(md5_file)

        if self.should_upload:
            self.upload()

    def generate_site_md5s(self):
        """generate MD5 checksums for all entities"""
        for root, dirs, files in os.walk(self.path):
            for name in files:
                # ignore hidden files
                if name.startswith("."):
                    continue

                filepath = os.path.join(root, name)

                with open(filepath, "rb") as infile:
                    raw = infile.read()
                    self.new_md5s[filepath] = md5(raw).hexdigest()

    def generate_redirect_md5s(self):
        """generate MD5 checkusm for all redirects"""
        for redirect in self.redirects:
            self.new_md5s[redirect] = md5(self.redirects[redirect]).hexdigest()

    def load_md5(self, filename):
        """
        load MD5 checksums from disk (json format)

        Arguments:
            filename: name of the file to load MD5 sums from
        """
        load = os.path.join(self.md5_path, filename)

        # if there is no MD5 file, just return
        if not os.path.isfile(load):
            return

        with open(load, "r", encoding="utf-8") as infile:
            raw = infile.read()
            self.old_md5s = json.loads(raw)

    def save_md5(self, filename):
        """
        save MD5 checksums to disk (json format)

        Arguments:
            filename: name of the file to save MD5 sums to
        """
        save = os.path.join(self.md5_path, filename)
        with open(save, "w", encoding="utf-8") as outfile:
            dumped = json.dumps(self.new_md5s)
            outfile.write(unicode(dumped))

    def compare_md5s(self):
        """compare MD5 checksums"""
        for key in self.new_md5s:
            if not key in self.old_md5s:
                self.changed.append(key)
                continue

            if self.new_md5s[key] != self.old_md5s[key]:
                self.changed.append(key)

    def upload(self):
        """upload changed files to S3"""
        for local in self.changed:
            remote = local.replace(self.path, self.s3path)

            proc = subprocess.Popen(
                [
                    "aws",
                    "s3",
                    "cp",
                    local,
                    remote,
                    "--profile",
                    self.profile
                ],
                cwd=self.path
            )
            proc.communicate()

    @property
    def should_upload(self):
        """
        Check if a changed file is part of self.skip_upload. If this
        is the case for all changed files do not upload the site.

        Returns:
            True if the site should be uploaded
        """
        upload = False

        for changed in self.changed:
            url = changed.replace(self.path, "")

            if not url in self.skip_upload:
                upload = True

        return upload

    def redirect(self):
        """create a redirect"""
        index = os.path.join(self.path, "index.html")

        for redirect in self.changed:
            # lstrip to make sure joining works if redirect starts with a /
            source = os.path.join(self.s3path, redirect.lstrip("/"))
            destination = urljoin(self.site_url, self.redirects[redirect])

            proc = subprocess.Popen(
                [
                    "aws",
                    "s3",
                    "cp",
                    index,
                    source,
                    "--website-redirect",
                    destination,
                    "--profile",
                    self.profile
                ],
                cwd=self.path
            )
            proc.communicate()



# -*- coding: utf-8 -*-
"""
    drupan.config

    Configuration for drupan. You can configure it by passing a dictionary
    with all values, passing a YAML formatted string or a path to a file.
"""

from io import open

import yaml


class Config(object):
    """hold every information needed by the engine and plugins to work"""
    def __init__(self):
        self.reader = None
        self.writer = None
        self.url_scheme = dict()
        self.plugins = None
        self.options = None
        self.deployment = None

    def from_file(self, cfg):
        """
        read the configuration from a  file

        Arguments:
            cfg: full path to configuration file
        """
        with open(cfg, 'r', encoding='utf-8') as infile:
            self.parse_yaml(infile.read())

    def get_option(self, section, key):
        """
        get a configuration option for a section of the system

        Arguments:
            section: plugin name e.x.
            key: option to get

        Returns:
            configuration option
        """
        sec = self.options.get(section, None)

        if not sec:
            message = "{0} improperly configured".format(section)
            raise Exception(message)

        opt = sec.get(key, None)

        if not opt:
            message = "could not find {0} for {1}".format(section, key)
            raise Exception(message)

        return opt

    def parse_yaml(self, raw):
        """
        parse yaml and configure this instance

        Arguments:
            raw: yaml data
        """
        parsed = yaml.load(raw)
        self.from_dict(parsed)

    def from_dict(self, cfg):
        """
        set instance variables form a dictionary

        Arguments:
            cfg: dictionary with configuration options
        """
        self.reader = cfg.get("reader", None)
        self.writer = cfg.get("writer", None)
        self.url_scheme = cfg.get("url_scheme", None)
        self.plugins = cfg.get("plugins", None)
        self.options = cfg.get("options", None)
        self.deployment = cfg.get("deployment", None)



# -*- coding: utf-8 -*-

"""
    drupan.serve

    Serve a generated site locally.
"""

import SimpleHTTPServer
import SocketServer
import os


def http(dir, port=9000):
    """
    Serve a directory over http.

    Arguments:
        dir: directory of the generated site
        port: port to run the httpd on
    """
    cwd = os.getcwd()
    os.chdir(dir)

    handler = SimpleHTTPServer.SimpleHTTPRequestHandler
    httpd = SocketServer.TCPServer(("", port), handler)

    try:
        print "server running on http://localhost:{0}".format(port)
        httpd.serve_forever()
    except KeyboardInterrupt:
        print "shutting down server\n"

    os.chdir(cwd)



# -*- coding: utf-8 -*-
"""
    drupan.plugins.blank

    Generate empty entities to be used for something that does not require
    its own content file.
"""

from drupan.entity import Entity


class Plugin(object):
    """create empty entities"""
    def __init__(self, site, config):
        """
        Arguments:
            site: instance of drupan.site.Site
            config: instance of drupan.config.Config
        """
        self.site = site
        self.config = config

        self.generate = config.get_option("blank", "generate")

    def run(self):
        """run the plugin"""
        for name in self.generate:
            new = Entity(self.config)
            new.meta["title"] = name
            new.meta["layout"] = self.generate[name]
            self.site.entities.append(new)



# -*- coding: utf-8 -*-
"""
    drupan.plugins.markdown

    Plugin that provides markdown conversion using markdown2
"""

from markdown2 import markdown


class Plugin(object):
    """convert entities content to markdown using markdown2"""
    def __init__(self, site, config):
        """
        Arguments:
            site: instance of drupan.site.Site
            config: instance of drupan.config.Config
        """
        self.site = site
        self.config = config

        self.extras = config.get_option("markdown", "extras")

    def run(self):
        """run the plugin"""
        for entity in self.site.entities:
            if entity.raw:
                self.convert(entity)

    def convert(self, entitiy):
        """
        convert entity.raw to markdown and save it to entity.content

        Arguments:
            entity: instance of drupan.entity.Entity
        """
        entitiy.content = markdown(entitiy.raw, extras=self.extras)



# -*- coding: utf-8 -*-

"""
    drupan.site

    Represent a site holding all information necessary to generate it and
    look up certain entities.
"""

from datetime import datetime


def search(entity, key, value):
    """
    Arguments:
        entity: drupan.Entity
        key: key to use
        value: value to match

    Returns:
        True if an entity has a attribute or key in meta with value as
        value
    """
    # TODO: comparison should be more flexible
    if key in entity.meta:
        if entity.meta[key] == value:
            return True

    if hasattr(entity, key):
        if getattr(entity, key) == value:
            return True

    return False


class Site(object):
    def __init__(self):
        self.entities = list()  # list for drupan.entity.Entity
        self.date = datetime.now()

    def get(self, key, value):
        """
        Filter entities and return one or more entities with a key that
        matches a given value.

        Arguments:
            key: key to look up - either a key in meta or attribute
            value: value to match

        Returns:
            Entity() or list with Entity()s
        """
        results = list()

        for entity in self.entities:
            if search(entity, key, value):
                results.append(entity)

        if len(results) > 1:
            return results
        elif len(results) == 1:
            return results[0]

        return None



# -*- coding: utf-8 -*-

from distutils.core import setup

from drupan.version import __version__


LONGDESC = """======
drupan - flexible static site generator
======
Drupan is a flexible static site generator helping you to create blogs, single
page applications or traditional websites. While being opionionated the plugin
system allows you to extend it with any functionallity you desire.

Noteworthy Features
======
  - deployment using git or directly to S3
  - fast generation time
  - Jinja2 with custom template tags and filters
  - powerful plugin system

Usage
======
You can install drupan via pip. To generate your site you just run
`drupan ~path/to/config.yaml`.

Supported command line switches

  - `nodeploy` do not deploy the generated site
  - `serve` runs a development server on port 9000
  - `init` creates a new site
  - `deploy` deploy your output directory without generation

Readers
======

  - `filesystem` reads content from files with a YAML header

Writers
======

  - `filesystem` writes the generated site to a directory

Deployment
======

  - `gitsub` commits the changes to git and pushes to a remote server
  - `s3sub` uploads the site to S3
  - `s3cf` upload directly to S3 and invalidate CouldFront using boto

Plugins
======

  - `blank` generates empty Entity instances with a given layout. This
  can be used to generate index or archive pages
  - `markdown` converts entity content from markdown to HTML
  - `tags` support for tags which are added to a posts meta information
"""


setup(
    name="drupan",
    version=__version__,
    author="Timo Zimmermann",
    author_email="timo@screamingatmyscreen.com",
    packages=[
        "drupan",
        "drupan/plugins",
        "drupan/deployment",
        "drupan/inout"
    ],
    url="http://pypi.python.org/pypi/drupan/",
    license="BSD",
    description="Drupan is a flexible static site generator helping you \
    to create blogs, single page applications or traditional websites.",
    long_description=LONGDESC,
    install_requires=[
        "pyyaml",
        "markdown2",
        "jinja2",
        "pygments",
    ],
    entry_points={
        "console_scripts": [
            "drupan = drupan.cmd:cmd"
        ]
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: BSD License",
        "Natural Language :: English",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX :: BSD :: FreeBSD",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Topic :: Internet",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content :: News/Diary",
        "Topic :: Internet :: WWW/HTTP :: Site Management",
        "Topic :: Software Development",
        "Topic :: Software Development :: Build Tools",
        "Topic :: Software Development :: Code Generators",
    ],
)
